-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Earth Strike",
	type = {"technique/arcane-earth", 1},
	require = techs_req1,
	points = 5,
	random_ego = "attack",
	cooldown = 8,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { pin = 2 } },
	getDuration = function(self,t) return math.floor(3+math.floor(self:getTalentLevel(t))) end,
	action = function(self, t)
		local shield = self:hasShield()
		--figure out which weapon (or none) to use
		local weapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			-- Try to pin !
			if hit then
				if target:canBe("pin") then
					target:setEffect(target.EFF_PINNED, t.getDuration(self,t), {apply_power=self:combatPhysicalpower()})
				else
					game.logSeen(target, "%s resists!", target.name:capitalize())
				end
			end
			--If wielding a shield, attack with it as a second weapon
			if shield then
				local speed, hit = self:attackTargetWith(target, shield.special_combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Even if we miss or no target, the power must explode in a ball!
		if self:getTalentLevel(t) >= 4 then
			local dur = t.getDuration(self,t)
			local chance = math.floor(self:getTalentLevel(t) * 5)
			self:project({type="ball", radius=math.floor(self:getTalentLevel(t)-3), selffire=false, friendlyfire=false}, x, y, DamageType.EARTH_STRIKE_BALL, {dur=dur,chance=chance})
		end
		if not self:isTalentActive(self.T_EARTH_STANCE) then self:forceUseTalent(self.T_EARTH_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true}) end
		return true
	end,
	--Learn and unlearn effects, add/remove earth stance
	--Increase/Decrease spell/earth mastery by 0.03
	on_learn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 1 then
			self:learnTalent(self.T_EARTH_STANCE, true, nil, {no_unlearn=true})
		end
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	end,
	on_unlearn = function(self, t)
		local lev = self:getTalentLevelRaw(t)
		if lev == 0 then
			self:unlearnTalent(self.T_EARTH_STANCE)
		end
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	end,

	info = function(self, t)
		local radius = math.max(math.floor(self:getTalentLevel(t)-3),0)
		local chance = math.floor(self:getTalentLevel(t) * 5)
		local dur = t.getDuration(self,t)
		return ([[Strike with the power of Earth. Pins target for %d turns and deals %d%% damage.  Chance to pin increases with Physical Power.
		At level 4 it blasts a ball of earth of radius %d which has a %d%% chance of pinning enemies as well.  After use, you change to the Earth Stance.]]):
		format(math.floor(3+self:getTalentLevel(t)),100 * self:combatTalentWeaponDamage(t, 1, 1.5),radius,chance)
	end,
}


newTalent{
	name = "Sandblast",
	type = {"technique/arcane-earth", 2},
	require = techs_req2,
	points = 5,
	random_ego = "attack",
	cooldown = 10,
	stamina = 12,
	tactical = { ATTACK = { weapon = 2 }, DISABLE = { blind = 2 } },
	getDuration = function(self,t) return math.floor(self:getTalentLevelRaw(t)) end,
	action = function(self, t)
		local shield = self:hasShield()
		--figure out which weapon (or none) to use
		local weapon = nil
		if self:getInven("MAINHAND") then
			weapon = self:getInven("MAINHAND")[1]
			if not weapon or not weapon.combat then
				weapon = nil
			end
		end
		if self:attr("disarmed") or not weapon then
			weapon = self.combat
		end

		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y then return nil end
		if target then
			if core.fov.distance(self.x, self.y, x, y) > 1 then return nil end
			local speed, hit = self:attackTargetWith(target, weapon, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			--If wielding a shield, attack with it as a second weapon
			if shield then
				local speed, hit = self:attackTargetWith(target, shield.special_combat, nil, self:combatTalentWeaponDamage(t, 1, 1.5))
			end
		end
		--Now blind effect!
		local dur = t.getDuration(self,t)
		local tgsand = {type="cone", radius=3, selffire=false, friendlyfire=false}
		self:project(tgsand, x, y, DamageType.SAND, {dam=self:getTalentLevel(t)*10,dur=dur})
		game.level.map:particleEmitter(self.x, self.y, tgsand.radius, "breath_earth", {radius=tgsand.radius, tx=x-self.x, ty=y-self.y})
		game:playSoundNear(self, "talents/earth")
		return true
	end,
	--Increase/Decrease spell/earth mastery by 0.03
	on_learn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	end,
	info = function(self, t)
		local dur = t.getDuration(self,t)
		return ([[Swing such that you strike the ground before you strike the target, sending a cone shaped blast of sand towards them.
		Your weapon deals %d%% damage.  The sand deals %0.2f damage and will blind all those affected for %d turns.]]):format(100 * self:combatTalentWeaponDamage(t, 1, 1.5),self:getTalentLevel(t)*10,dur)
	end,
}

newTalent{
	name = "Earthen Shield",
	type = {"technique/arcane-earth", 3},
	require = techs_req3,
	points = 5,
	mode = "sustained",
	sustain_mana = 50,
	cooldown = 30,
	range = function(self, t) return 1 + self:getTalentLevelRaw(t) end,
	tactical = { DEFEND = 2 },
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 25, 500) end,
	getRegenPercent = function(self,t) return math.min(self:getTalentLevelRaw(t)*2,10) end,
	do_earthenShieldRegen = function(self, t)
		game.logSeen("Earthen shield absorb currently: %d , regen is %d",self.earthen_shield_absorb,self.earthen_shield_regen)
		self.earthen_shield_absorb = math.min(self.earthen_shield_absorb + self.earthen_shield_regen,self.earthen_shield_max)
		game.logSeen("Earthen shield is now %d",self.earthen_shield_absorb)
	end,
	activate = function(self, t)
		local power = t.getDamage(self, t)
		self.earthen_shield_absorb = power
		self.earthen_shield_regen = power * t.getRegenPercent(self,t) / 100
		self.earthen_shield_max = power
		game:playSoundNear(self, "talents/generic")
		return {
			shield = self:addTemporaryValue("earthen_shield", power),
		}
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("earthen_shield", p.shield)
		self.earthen_shield_absorb = nil
		self.earthen_shield_regen = nil
		self.earthen_shield_max = nil
		return true
	end,
	--Increase/Decrease spell/earth mastery by 0.03
	on_learn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	end,
	info = function(self, t)
		local shield_max = 0
		local shield_absorb = t.getDamage(self,t)
		local regenPercent = t.getRegenPercent(self,t)
		if self.earthen_shield_absorb and self.earthen_shield_regen and self.earthen_shield_max then
			shield_max = self.earthen_shield_max
			shield_absorb = self.earthen_shield_absorb
		end
		return ([[Covers you in rock and stone, negating half of all damage you take.  Earthen Shield has a pool of %0.2f/%0.2f hit points, which when it reaches zero causes the it to crumble away, deactivated.
		Each turn Earthen Shield will regenerate %d%% of its maximum shield point pool.
		The amount absorbed will increase with spell power.]]):format(shield_absorb,shield_max,regenPercent)
	end,
}


newTalent{
	name = "Rolling Earth",
	type = {"technique/arcane-earth", 4},
	require = techs_req4,
	points = 5,
	mana = 40,
	cooldown = 20,
	tactical = { DISABLE = { stun = 3 }, CLOSEIN = 2  },
	range = 0,
	requires_target = true,
	radius = function(self, t)
		return 1 + self:getTalentLevelRaw(t)
	end,
	--target = function(self, t)
	--	return {type="cone", range=self:getTalentRange(t), friendlyfire=false, radius=self:getTalentRadius(t), talent=t}
	--end,

	target = function(self, t)
		return {type="cone", range=self:getTalentRange(t), friendlyfire=false, radius=self:getTalentRadius(t), talent=t}
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, function(px, py)
			local target = game.level.map(px, py, Map.ACTOR)
			if not target then return end
			local tx, ty = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
			if tx and ty and target:canBe("knockback") then
				target:move(tx, ty, true)
				game.logSeen(target, "%s is tossed by the earth!", target.name:capitalize())
			end
			if target:canBe("stun") then
				target:setEffect(target.EFF_STUNNED, 3 + self:getTalentLevelRaw(t), {apply_power=self:combatSpellpower()})
			else
				game.logSeen(target, "%s resists being stunned!", target.name:capitalize())
			end
		end)
		game.level.map:particleEmitter(x, y, tg.radius, "gravity_spike", {radius=tg.radius, grids=grids, tx=self.x, ty=self.y})
		game:playSoundNear(self, "talents/earth")
		return true
	end,


	--action = function(self, t)
	--	local tg = self:getTalentTarget(t)
	--	local x, y = self:getTarget(tg)
	--	if not x or not y then return nil end
	--	local grids = self:project(tg, self.x, self.y, function(px, py)
	--		game.logSeen(self,"self defined project entered")
	--		local target = game.level.map(px, py, Map.ACTOR)
	--		if not target then return end
	--		local tx, ty = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
	--		if tx and ty then game.logSeen(self,"X and Y are: %d , %d",tx,ty) end
	--		if tx and ty and target:canBe("knockback") then
	--			target:move(tx, ty, true)
	--			game.logSeen(target, "%s is tossed closer by the rolling earth!", target.name:capitalize())
	--		end
	--	end)

	--	return true
	--end,
	--Increase/Decrease spell/earth mastery by 0.03
	on_learn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") + 0.03)
	end,
	on_unlearn = function(self, t)
		self:setTalentTypeMastery("spell/earth", self:getTalentTypeMastery("spell/earth") - 0.03)
	end,
	info = function(self, t)
		return ([[Causes the ground to shake violently in such a way as to toss those in it towards the caster, stunning them for %d rounds in the process.
		Affects a cone of radius %d.  Chance to stun increases with spell power.]]):format(3 + self:getTalentLevelRaw(t), 1 + self:getTalentLevelRaw(t))
	end,
}

newTalent{
	name = "Earth Stance",
	type = {"technique/elemental-stance", 1},
	points = 1,
	mode = "sustained",
	cooldown = 0,
	no_energy = true,
	elemental_stance = true,
	getStanceArmorMod = function(self,t) return math.ceil((self:combatSpellpower() / 10.0) * (1 +self:getTalentLevel(self.T_ARCANE_STANCE))) end,
	getStanceResistMod = function(self,t) return (self:combatSpellpower() / 50.0) * (1 + self:getTalentLevel(self.T_ARCANE_STANCE)) end,
	on_pre_use = function(self, t, silent)
		-- prevent anyone from activating other than without the "force_talent_ignore_ressources" flag
		if not self:attr("force_talent_ignore_ressources") then
			if not silent then game.logPlayer(self, "You can not active Earth Stance directly.  Use Earth Strike to activate it.") end
			return false
		end
		return true
	end,
	activate = function(self,t)
		local stance_turned_off = nil
		self:attr("earth_stance", 1)
		if self:isTalentActive(self.T_WATER_STANCE) then
			self:forceUseTalent(self.T_WATER_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_AIR_STANCE) then
			self:forceUseTalent(self.T_AIR_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if self:isTalentActive(self.T_FIRE_STANCE) then
			self:forceUseTalent(self.T_FIRE_STANCE, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			stance_turned_off = 1
		end
		if not stance_turned_off and self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		return {
			armor = self:addTemporaryValue("combat_armor", t.getStanceArmorMod(self,t)),
			res = self:addTemporaryValue("resists", {all=t.getStanceResistMod(self,t)})
		}
	end,
	deactivate = function(self,t,p)
		self:attr("earth_stance",-1)
		--turn Arcane Destruction on and off if on.
		if self:isTalentActive(self.T_ARCANE_DESTRUCTION) then
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
			self:forceUseTalent(self.T_ARCANE_DESTRUCTION, {ignore_energy=true, ignore_ressources=true, ignore_cd =true})
		end
		self:removeTemporaryValue("combat_armor", p.armor)
		self:removeTemporaryValue("resists", p.res)
		return true
	end,
	info = function(self, t)
		local armor = t.getStanceArmorMod(self,t)
		local res = t.getStanceResistMod(self,t)
		return ([[Enter the elemental stance of Earth by using the Earth Strike talent.  Earth Stance increases your armor by %d and your resistance to all damage by %0.2f.
		Benefits increase with the Arcane Stance talent.]])
		:format(armor, res)
	end,
}
